import { randomUUID } from "crypto";

export interface LedgerEntry {
    timestamp: number,
    type: ('post' | 'citation'),
    postId: string,
    citationId: string | null,
    change: number,
    balance: number
}

export class Member {

    static localStore: Map<string, Member>;

    private id: string;
    private reputation: number;
    private ledger: LedgerEntry[];

    constructor(id?: string) {
        this.id = id ? id : randomUUID();
        this.reputation = 0;
        this.ledger = [];
        Member.localStore.set(this.id, this);
    }
    
    public static getAllMembers() {
        return Array.from(Member.localStore.values());
    }

    public static getMember(id: string) {
        return Member.localStore.get(id);
    }

    public getId() { return this.id; }
    public getReputation() { return this.reputation; }
    public getLedger() { return this.ledger; }

    public postReputation(postId: string, amount: number) {
        const entry: LedgerEntry = {
            timestamp: Date.now(),
            type: "post",
            postId: postId,
            citationId: null,
            change: amount,
            balance: this.reputation + amount
        }
        this.reputation = entry.balance;
        return entry;
    }

    public citationReputation(postId: string, citationId: string, amount: number) {
        const entry: LedgerEntry = {
            timestamp: Date.now(),
            type: "citation",
            postId: postId,
            citationId: citationId,
            change: amount,
            balance: this.reputation + amount
        }
        this.reputation = entry.balance;
        return entry;
    }
    
}